/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.junit.tools;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.meldungserstellung.VerwaltungMeldungen;

public class WarteAufAktualisierteDaten implements ClientReceiverInterface 
{
  private Collection<SystemObject> _objekte = new LinkedList<SystemObject>();
  
  private List<SystemObject> _angemeldeteObjekte = new LinkedList<SystemObject>();
  
  private Map<String, ResultData> _aktualisierteDaten = new HashMap<String, ResultData>();
  
  public Map<String, ResultData> getAktualisierteDaten()
  {
    return Collections.unmodifiableMap(_aktualisierteDaten);
  }
  
  private Map<String, ResultData> _initialeDaten = new HashMap<String, ResultData>();
  
  /**
   * Es werden alle Objekte einer Menge bestimmt, deren Daten aktualisiert werden (Aktualisierung
   * von einem gltigen Datensatz auf einen aktualisierten gltigen Datensatz). Auf Aktualisiereungen 
   * wird gewartet, bis alle Objekte, fr die Aktualisierungen erwartet werden, aktualisiert sind oder
   * bis ein Timeout verstrichen ist.
   * 
   * @param connection die DAV-Verbindung
   * @param menge die Menge, deren Objekte berwacht wird 
   * @param objekte die Objekte, fr die Aktualisierungen erwartet werden
   * @param dd Datenbeschreibung der erwarteten aktualisierten Datenstze
   * @param timeout Timeout - solange wird maximal auf aktualisierte gltige Daten fr alle Objekte 
   * gewartet - wird hier 0 bergeben, wird unendlich lange gewartet (dead lock Gefahr)
   */
  public WarteAufAktualisierteDaten(ClientDavInterface connection, MutableSet menge, 
      SystemObject[] objekte, DataDescription dd, long timeout)
  {
    List<SystemObject> elemente = menge.getElements();
    
    for (SystemObject objekt: objekte)
    {
      if (!objekt.getPid().startsWith(VerwaltungMeldungen.PID_PREFIX))
      {
        throw new IllegalArgumentException("Die PIDs der bergebenen Objekte mssen mit " + 
            VerwaltungMeldungen.PID_PREFIX + " beginnen");
      }
      if (!elemente.contains(objekt))
      {
        throw new IllegalArgumentException("Die bergebenen Objekte mssen Elemente der bergebenen Menge " + 
            menge.getName() + " sein");
      }
      _objekte.add(objekt);
    }
    
    for (SystemObject objekt: elemente)
    {
      if (objekt.getPid().startsWith(VerwaltungMeldungen.PID_PREFIX))
      {
        connection.subscribeReceiver(this, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        _angemeldeteObjekte.add(objekt);
      }
    }
    
    try
    {
      synchronized (this)
      {
          wait(timeout);
      }
    }
    catch(InterruptedException e)
    {
      throw new RuntimeException("Unerwarteter Abbruch", e);
    }
    finally
    {
      connection.unsubscribeReceiver(this, _angemeldeteObjekte, dd);
    }
  }
      
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] resultData)
  {
    for (ResultData data: resultData)
    {
      if (data.hasData())
      {
        String pid = data.getObject().getPid();
        if (_initialeDaten.get(pid) == null) // initiale Daten empfangen
        {  
          _initialeDaten.put(pid, data);
        }
        else if (_aktualisierteDaten.get(pid) == null) // initiale Daten bereits vorhanden
        {                                              // dies ist die Aktualisierung       
          _aktualisierteDaten.put(pid, data);
        }
      }
    }
    if (_aktualisierteDaten.size() >= _objekte.size())
    {
      synchronized(this)
      {
        notify();
      }
    }
  }
  
}
