/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */


package de.bsvrz.ste.rdstmc.konfiguration;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * <code>Konfiguration</code> verwaltet Konfigurationsdaten - Straensegmente
 * und -Knoten, Messquerschnitte und Straenabschnitte.
 * Die Klasse ist  genau ein Konfigurationsobjekt vom Typ typ.netz und fr eine 
 * DAV-Verbindung instanziierbar.
 * Bei der Instanziierung werden die Straensegmente und -Knoten, Messquerschnitte 
 * und Straenabschnitte fr das bergebene Netz geladen und fr diese Objekttypen 
 * wird das DAV-Datenmodell abgebildet
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.2 $ / $Date: 2009/01/27 13:54:21 $ / ($Author: Sans $)
 */
public class Konfiguration
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
   
  /**
   * Erzeugt eine Instanz vom Typ <code>Konfiguration</code> fr eine DAV-Verbindung
   * und ein Konfigurationsobjekt vom Typ typ.netz
   * 
   * @param connection die  Datenveteiler-Verbindung
   * @param netz das (Konfigurations-)Obejkt, dessen Straensegmente und -Knoten, 
   * Messquerschnitte und Straenabschnitte ausgewertet werden
   * @param nurKorrekteTmcInfo wenn <code>true</code> werden uere Strassensegmente mit
   * fehlerhafter oder unvollstngen TMC-Information nicht bernommen
   * @param keineInnerenSegmenteAussen wenn <code>true</code> werden innere Strassensegmente, die 
   * "aussen" liegen (d. h. deren Vorgnger/Nachfolger nicht mehr im Netz liegt) nicht bernommen
   */
  public Konfiguration(ClientDavInterface connection, ConfigurationObject netz, 
      boolean nurKorrekteTmcInfo, boolean keineInnerenSegmenteAussen)
  {
    _connection = connection;
    _netz = netz;
    
    _strassenKnoten = new HashMap<String, StrassenKnoten>();
    _tmcPunkte = new HashMap<String, TmcLocationPunkt>();
    _tmcPunkteMitLocationCode = new HashMap<String, TmcLocationPunkt>();
    
    _aeussereStrassenSegmente = bestimmeAeussereStrassenSegmente(connection, netz, nurKorrekteTmcInfo);
    _debug.config(netz.getPid() + ": " + _aeussereStrassenSegmente.size() + " uere(s) Straensegment(e) mit " + 
        _tmcPunkte.size() + " TMC-Punkt(en) und " + _strassenKnoten.size() + " Straenknoten werden bearbeitet");
        
    _innereStrassenSegmente = bestimmeInnereStrassenSegmente(connection, _aeussereStrassenSegmente, keineInnerenSegmenteAussen);
    _debug.config(netz.getPid() + ": " + _innereStrassenSegmente.size() + " innere(s) Straensegment(e) werden bearbeitet");
    
    _strassenTeilSegmente = bestimmeStrassenTeilSegmente(connection, _aeussereStrassenSegmente);
    _strassenTeilSegmente.putAll(bestimmeStrassenTeilSegmente(connection, _innereStrassenSegmente));
    _debug.config(netz.getPid() + ": " + _strassenTeilSegmente.size() + " Straenteilsegment(e) werden bearbeitet");
    
    _messQuerschnitte = bestimmeMessQuerschnitte(connection, _aeussereStrassenSegmente);
    _messQuerschnitte.putAll(bestimmeMessQuerschnitte(connection, _innereStrassenSegmente));
    _debug.config(netz.getPid() + ": " + _messQuerschnitte.size() + " Messquerschnitt(e) werden bearbeitet");

    _strassenAbschnitte = bestimmeStrassenAbschnitte(connection, _messQuerschnitte);
    _debug.config(netz.getPid() + ": " + _strassenAbschnitte.size() + " Straenabschnitt(e) werden bearbeitet");
  }
  
  /**
   * Erzeugt eine Instanz vom Typ <code>Konfiguration</code> fr eine DAV-Verbindung
   * und ein Konfigurationsobjekt vom Typ typ.netz
   * 
   * @param connection die  Datenveteiler-Verbindung
   * @param netz das (Konfigurations-)Obejkt, dessen Straensegmente und -Knoten, 
   * Messquerschnitte und Straenabschnitte ausgewertet werden
   */
  public Konfiguration(ClientDavInterface connection, ConfigurationObject netz)
  {
    this(connection, netz, false, false);
  }
  
  /**
   * DAV-Verbindung
   */
  private ClientDavInterface _connection;
  
  /**
   * 
   * @return die DAV-Verbindung
   */
  private ClientDavInterface getConnection()
  {
    return _connection;
  }
  
  /**
   * Konfigurationsobjekt vom Typ typ.netz
   */
  private ConfigurationObject _netz;
  
  /**
   * 
   * @return das Konfigurationsobjekt vom Typ typ.netz
   */
  public ConfigurationObject getNetz()
  {
    return _netz;
  }
  
  /**
   * Die Straenknoten
   */
  private Map<String, StrassenKnoten> _strassenKnoten;
  
  /**
   * Read-only Zugriff auf die Straenknoten
   * 
   * @return die Straenknoten
   */
  public Map<String, StrassenKnoten> getStrassenKnoten()
  {
    return Collections.unmodifiableMap(_strassenKnoten);
  }
  
  /**
   * Die Tmc-Punkte
   */
  private Map<String, TmcLocationPunkt> _tmcPunkte;
  
  /**
   * Read-only Zugriff auf die Tmc-Punkte
   * 
   * @return Zuordnung der Tmc-Punkte zu ihrer PID
   */
  public Map<String, TmcLocationPunkt> getTmcPunkte()
  {
    return Collections.unmodifiableMap(_tmcPunkte);
  }
  
  /**
   * Die Tmc-Punkte, Zugriff ber Locationcode
   */
  private Map<String, TmcLocationPunkt> _tmcPunkteMitLocationCode;
  
  /**
   * Read-only Zugriff auf die Tmc-Punkte mit Zugriff ber Locationcode
   * 
   * @return Zuordnung der Tmc-Punkte zu ihrem Locationcode
   */
  public Map<String, TmcLocationPunkt> getTmcPunkteMitLocationCode()
  {
    return Collections.unmodifiableMap(_tmcPunkteMitLocationCode);
  }
  
  /**
   * Die ueren Straensegmente
   */
  private Map<String, AeusseresStrassenSegment> _aeussereStrassenSegmente;
  
  /**
   * Read-only Zugriff auf die ueren Straensegmente
   * 
   * @return die ueren Straensegmente
   */
  public Map<String, AeusseresStrassenSegment> getAeussereStrassenSegmente()
  {
    return Collections.unmodifiableMap(_aeussereStrassenSegmente);
  }
  
  /**
   * Die inneren Straensegmente
   */
  private Map<String, InneresStrassenSegment> _innereStrassenSegmente;
  
  /**
   * Read-only Zugriff auf die inneren Straensegmente
   * 
   * @return die inneren Straensegmente
   */
  public Map<String, InneresStrassenSegment> getInnereStrassenSegmente()
  {
    return Collections.unmodifiableMap(_innereStrassenSegmente);
  }
  
  /**
   * Die relevanten Messquerschnitte
   */
  private Map<String, MessQuerschnitt> _messQuerschnitte;
  
  /**
   * Read-only Zugriff auf die Messquerschnitte
   * 
   * @return die relevanten Messquerschnitte
   */
  public Map<String, MessQuerschnitt> getMessQuerschnitte()
  {
    return Collections.unmodifiableMap(_messQuerschnitte);
  }
  
  /**
   * Die relevanten Strassenteilsegmente
   */
  private Map<String, StrassenTeilSegment> _strassenTeilSegmente;
  
  /**
   * Read-only Zugriff auf die Strassenteilsegmente
   * 
   * @return die relevanten Strassenteilsegmente
   */
  public Map<String, StrassenTeilSegment> getStrassenTeilSegmente()
  {
    return Collections.unmodifiableMap(_strassenTeilSegmente);
  }
  
  /**
   * Die relevanten Straenabschnitte
   */
  private Map<String, StrassenAbschnitt> _strassenAbschnitte;
  
  /**
   * Read-only Zugriff auf die Messquerschnitte
   * 
   * @return die relevanten Straenabschnitte
   */
  public Map<String, StrassenAbschnitt> getStrassenAbschnitte()
  {
    return Collections.unmodifiableMap(_strassenAbschnitte);
  }
   
  /**
   * Bestimmt alle usseren Strassensegmente im bergebenen Netz und dessen Unternetzen
   * 
   * @param netz das (Wurzel-) Netz, in dem die usseren Strassensegmente bestimmt werden
   * @return Liste alle gefundenen usseren Strassensegmente
   */
  private Map<String, AeusseresStrassenSegment> bestimmeAeussereStrassenSegmente(ClientDavInterface connection,
      ConfigurationObject netz, boolean nurKorrekteTmcInfo)
  {
    if (!netz.isOfType("typ.netz"))
    {
      throw new IllegalArgumentException("Objekt " + netz + " ist nicht vom Typ typ.netz");
    }
    
    NonMutableSet menge = netz.getNonMutableSet("NetzBestandTeile");
    if (menge != null)  
    {  
      Map<String, AeusseresStrassenSegment> result = new HashMap<String, AeusseresStrassenSegment> ();

      for (SystemObject obj: menge.getElements())
      {
        if (obj.isOfType("typ.ueresStraenSegment"))
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.straenSegment");
          
          Data data = obj.getConfigurationData(atg);
          if (data != null)
          {
            long laenge = data.getItem("Lnge").asUnscaledValue().longValue() / 100;
            
            SystemObject strasse = data.getItem("gehrtZuStrae").asReferenceValue().getSystemObject();
            String nummer = null;
            if (strasse != null)
            {
              atg = connection.getDataModel().getAttributeGroup("atg.strae");
              data = strasse.getConfigurationData(atg);
              if (data != null)
              {
                nummer = data.getItem("Nummer").asTextValue().getText();
              }
              _debug.finest("Straennummer von " + obj.getPid() + ": " + nummer);
            }
            atg = connection.getDataModel().getAttributeGroup("atg.ueresStraenSegment");
            data = obj.getConfigurationData(atg);
            if (data != null)
            {
              SystemObject von = data.getItem("vonKnoten").asReferenceValue().getSystemObject();
              SystemObject nach = data.getItem("nachKnoten").asReferenceValue().getSystemObject();
              
              int tmcRichtung = data.getItem("TmcRichtung").asUnscaledValue().intValue();
              SystemObject tmcPrimaerOrt = data.getItem("TmcPrimrOrt").asReferenceValue().getSystemObject();
              TmcLocationPunkt tmcPunkt = null;
              
              if (tmcPrimaerOrt != null)
              {  
                tmcPunkt = _tmcPunkte.get(tmcPrimaerOrt.getPid());
                if (tmcPunkt == null)
                {  
                  tmcPunkt = new TmcLocationPunkt(tmcPrimaerOrt, connection.getDataModel());
                  _tmcPunkte.put(tmcPrimaerOrt.getPid(), tmcPunkt);
                  _tmcPunkteMitLocationCode.put("" + tmcPunkt.getLocationCode(), tmcPunkt);
                }
                _debug.finest("TMC Location Code von " + obj.getPid() + ": " + tmcPunkt.getLocationCode());
                
                if (tmcPunkt.getVorgaenger() == null || tmcPunkt.getNachfolger() == null)
                {
                  atg = connection.getDataModel().getAttributeGroup("atg.tmcPunkt");
                  data = tmcPrimaerOrt.getConfigurationData(atg);
                  
                  if (tmcPunkt.getVorgaenger() == null)
                  {
                    SystemObject vorg = data.getItem("Vorgnger").asReferenceValue().getSystemObject();
                    if (vorg != null)
                    {  
                      TmcLocationPunkt vorgaenger = _tmcPunkte.get(vorg.getPid());
                      if (vorgaenger == null)
                      {
                        vorgaenger = new TmcLocationPunkt(vorg, connection.getDataModel());
                        _tmcPunkte.put(vorg.getPid(), vorgaenger);
                        _tmcPunkteMitLocationCode.put("" + vorgaenger.getLocationCode(), vorgaenger);
                      }
                      tmcPunkt.setVorgaenger(vorgaenger);
                      if (vorgaenger.getNachfolger() == null)
                      {  
                        vorgaenger.setNachfolger(tmcPunkt);
                      }
                      else if (vorgaenger.getNachfolger() != tmcPunkt)
                      {
                        _debug.warning("Inkonsistente TMC-Location Daten: TMC-Knoten " + vorgaenger.getLocationCode() +
                            " wurde als Vorgnger von " + tmcPunkt.getLocationCode() + " bestimmt, hat aber bereits " +
                            vorgaenger.getNachfolger().getLocationCode() + " als Nachfolger");
                      }
                    }  
                  }
                  if (tmcPunkt.getNachfolger() == null)
                  {
                    SystemObject nachf = data.getItem("Nachfolger").asReferenceValue().getSystemObject();
                    if (nachf != null)
                    {  
                      TmcLocationPunkt nachfolger = _tmcPunkte.get(nachf.getPid());
                      if (nachfolger == null)
                      {
                        nachfolger = new TmcLocationPunkt(nachf, connection.getDataModel());
                        _tmcPunkte.put(nachf.getPid(), nachfolger);
                        _tmcPunkteMitLocationCode.put("" + nachfolger.getLocationCode(), nachfolger);
                      }
                      tmcPunkt.setNachfolger(nachfolger);
                      if (nachfolger.getVorgaenger() == null)
                      {
                        nachfolger.setVorgaenger(tmcPunkt);
                      }
                      else if (nachfolger.getVorgaenger() != tmcPunkt)
                      {
                        _debug.warning("Inkonsistente TMC-Location Daten: TMC-Knoten " + nachfolger.getLocationCode() +
                            " wurde als Nachfolger von " + tmcPunkt.getLocationCode() + " bestimmt, hat aber bereits " +
                            nachfolger.getVorgaenger().getLocationCode() + " als Vorgaenger");
                      }
                    }  
                  }
                }  
              }  
              
              if (!nurKorrekteTmcInfo || (tmcPunkt != null && tmcRichtung != 0))
              {  
                AeusseresStrassenSegment segment = new AeusseresStrassenSegment(obj, laenge, 
                    tmcPunkt, tmcRichtung, nummer);
                              
                if (von != null)
                {
                  StrassenKnoten knoten = _strassenKnoten.get(von.getPid());
                  if (knoten == null)
                  {
                    atg = connection.getDataModel().getAttributeGroup("atg.straenKnoten");
                    data = von.getConfigurationData(atg);
                    if (data != null)
                    {
                      String typ = data.getItem("Typ").asTextValue().getText();
                      knoten = new StrassenKnoten(von, typ);
                      _strassenKnoten.put(von.getPid(), knoten);
                    }
                  }
                  segment.setVonKnoten(knoten);
                  knoten.addAbgehendesSegment(segment);
                }
                
                if (nach != null)
                {
                  StrassenKnoten knoten = _strassenKnoten.get(nach.getPid());
                  if (knoten == null)
                  {
                    atg = connection.getDataModel().getAttributeGroup("atg.straenKnoten");
                    data = nach.getConfigurationData(atg);
                    if (data != null)
                    {
                      String typ = data.getItem("Typ").asTextValue().getText();
                      knoten = new StrassenKnoten(nach, typ);
                      _strassenKnoten.put(nach.getPid(), knoten);
                    }
                  }
                  segment.setNachKnoten(knoten);
                  knoten.addHinfhrendesSegment(segment);
                }
                
                if ( true /* obj.getPid().equals("ass.p.11905.p.11906.0")  || 
                     obj.getPid().equals("ass.n.11983.n.11982.0") */)
                {  
                  result.put(obj.getPid(), segment);
                }  
              }
              else
              {
                _debug.warning("Korrekte TMC-Daten (Ort und/oder Richtung ) nicht versorgt fr " + obj + 
                    "\n=> KANN NICHT VERWENDET WERDEN");
              }
            }
            else
            {
              _debug.warning("Daten von " + atg + " nicht versorgt fr " + obj + "\n=> KANN NICHT VERWENDET WERDEN");              
            }
          }
          else
          {
            _debug.warning("Daten von " + atg + " nicht versorgt fr " + obj + "\n=> KANN NICHT VERWENDET WERDEN");
          }
        }
        else if (obj.isOfType("typ.netz"))
        { 
          result.putAll(bestimmeAeussereStrassenSegmente(connection, (ConfigurationObject)obj, nurKorrekteTmcInfo));
        }
        else
        {
          _debug.warning("Objekt " + obj + " mit unerwartetem Typ " + obj.getType() + 
              " gefunden in der Menge \"NetzBestandTeile\" von " + netz);
        }
      }
      
      _debug.fine(result.size() + " ussere(s) Straensegment(e) bestimmt in (Teil-) Netz " + netz.getPidOrId());
      return result;
    }
    else
    {
      throw new RuntimeException("Menge \"NetzBestandteile\" nicht vorhanden in " + netz);
    }
  }
  
  /**
   * Bestimmt alle inneren Straensegmente  auf einer Menge von ueren Straensegmenten
   * 
   * @param connection DAV-Verbindung
   * @param aeussereStrassenSegmente ussere Straensegmente, zu dennen die inneren Straensegmente 
   * bestimmt werden sollen 
   * @return Sammlung aller gefundenen innerern Straensegmente
   */
  private Map<String, InneresStrassenSegment> bestimmeInnereStrassenSegmente(
      ClientDavInterface connection, Map<String, AeusseresStrassenSegment> aeussereStrassenSegmente,
        boolean keineInnerenSegmenteAussen)
  { 
    Map<String, InneresStrassenSegment> result = new HashMap<String, InneresStrassenSegment> ();
    SystemObjectType type = connection.getDataModel().getType("typ.inneresStraenSegment");

    AttributeGroup atgSS = connection.getDataModel().getAttributeGroup("atg.straenSegment");
    AttributeGroup atgISS = connection.getDataModel().getAttributeGroup("atg.inneresStraenSegment");
       
    connection.getDataModel().getConfigurationData(type.getElements(), atgSS);
    connection.getDataModel().getConfigurationData(type.getElements(), atgISS);

    for (SystemObject obj: type.getElements())
    {
      if (obj.isValid())
      {
        Data data = obj.getConfigurationData(atgSS);
        
        if (data != null)
        {
          long laenge = data.getItem("Lnge").asUnscaledValue().longValue() / 100;
          
          data = obj.getConfigurationData(atgISS);
          if (data != null)
          {
            SystemObject von = data.getItem("vonStraenSegment").asReferenceValue().getSystemObject();
            SystemObject nach = data.getItem("nachStraenSegment").asReferenceValue().getSystemObject();
            AeusseresStrassenSegment vonSegment = von != null ?
                aeussereStrassenSegmente.get(von.getPid()) : null;
            AeusseresStrassenSegment nachSegment = nach != null ?
                aeussereStrassenSegmente.get(nach.getPid()) : null;    
                
            if ((!keineInnerenSegmenteAussen && (nachSegment != null || vonSegment != null)) ||
              (nachSegment != null && vonSegment != null))
            {
              InneresStrassenSegment segment = new InneresStrassenSegment(obj, laenge);    
              
              if (nachSegment != null)
              {  
                nachSegment.addVorgaenger(segment);
                segment.addNachfolger(nachSegment);
              }
              if (vonSegment != null)
              {
                vonSegment.addNachfolger(segment);
                segment.addVorgaenger(vonSegment);
              }
              
              if (nachSegment != null && vonSegment != null && 
                  nachSegment.liegtAufSelberStrasse(vonSegment))
              {
                segment.addVorgaengerAufStrasse(vonSegment);
                vonSegment.addNachfolgerAufStrasse(segment);
                segment.addNachfolgerAufStrasse(nachSegment);
                nachSegment.addVorgaengerAufStrasse(segment);
              }
              
              StrassenKnoten strassenKnoten;
              if (nachSegment != null)
              {  
                strassenKnoten = nachSegment.getVonKnoten(); 
              }
              else
              {
                strassenKnoten = vonSegment.getNachKnoten();
              }
              strassenKnoten.addInneresSegment(segment);
              
              result.put(obj.getPid(), segment);
            }
          }  
          else
          {
            _debug.warning("Daten von " + atgISS + " nicht versorgt fr " + obj + "\n=> KANN NICHT VERWENDET WERDEN");              
          }
        }
        else
        {
          _debug.warning("Daten von " + atgSS + " nicht versorgt fr " + obj + "\n=> KANN NICHT VERWENDET WERDEN");
        }
      }
    }
    
    return result;
  }
    
  /**
   * Bestimmt alle Messquerschnitte auf einer Menge von Strassensegmenten
   * 
   * @param connection DAV-Verbindung
   * @param strassenSegmente Sammlung von Straensegmente, deren MQ bestimmt werden sollen 
   * @return Sammlung aller gefundenen MQ
   */
  private Map<String, MessQuerschnitt> bestimmeMessQuerschnitte(ClientDavInterface connection,
      Map<String, ? extends StrassenSegment> strassenSegmente)
  { 
    Map<String, MessQuerschnitt> result = new HashMap<String, MessQuerschnitt> ();
    SystemObjectType type = connection.getDataModel().getType("typ.messQuerschnittAllgemein");
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.punktLiegtAufLinienObjekt");
    
    for (SystemObject obj: type.getElements())
    {
      if (obj.isValid())
      {
        Data data = obj.getConfigurationData(atg);
        if (data != null)
        {
          ReferenceValue linienRef = data.getItem("LinienReferenz").asReferenceValue();
          StrassenSegment segment = strassenSegmente.get(linienRef.getSystemObject().getPid()); 
          if (segment != null)
          {
            long offset = data.getItem("Offset").asUnscaledValue().longValue() / 100;
            MessQuerschnitt mq = new MessQuerschnitt(obj, segment, offset);
            result.put(obj.getPid(), mq);
            segment.addMessQuerschnitt(mq);
          }
        }
      }
    }
    
    return result;
  }
  
  /**
   * Bestimmt alle Strassenteilsegmente einer Menge von Strassensegmenten
   * 
   * @param connection DAV-Verbindung
   * @param strassenSegmente Sammlung von Straensegmente, deren Strassenteilsegmente bestimmt werden sollen 
   * @return Sammlung aller gefundenen Strassenteilsegmente
   */
  private Map<String, StrassenTeilSegment> bestimmeStrassenTeilSegmente(ClientDavInterface connection,
      Map<String, ? extends StrassenSegment> strassenSegmente)
  {
    Map<String, StrassenTeilSegment> result = new HashMap<String, StrassenTeilSegment> ();
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.bestehtAusLinienObjekten");
    
    for (StrassenSegment segment: strassenSegmente.values())
    {
      Data data = segment.getDavObjekt().getConfigurationData(atg);
      
      if (data != null)
      {
        AttributeGroup atgSts = connection.getDataModel().getAttributeGroup("atg.straenTeilSegment");
        ReferenceArray array = data.getItem("LinienReferenz").asReferenceArray();
        long offset = 0;
        
        for (ReferenceValue ref: array.getReferenceValues())
        {
          SystemObject val = ref.getSystemObject();
          if (val.isOfType("typ.straenTeilSegment"))
          {
            Data dataSts = val.getConfigurationData(atgSts);
            if (dataSts != null)
            {
              long laenge = dataSts.getItem("Lnge").asUnscaledValue().longValue() / 100;
              int anzahlFahrStreifen = dataSts.getItem("AnzahlFahrStreifen").asUnscaledValue().intValue();
              String steigungGefaelle = dataSts.getItem("SteigungGeflle").asTextValue().getText();
              
              StrassenTeilSegment sts = new StrassenTeilSegment(val, laenge, anzahlFahrStreifen, 
                  steigungGefaelle, offset);
              segment.addStrassenTeilSegment(sts);
              offset += laenge;
              result.put(sts.getDavObjekt().getPid(), sts);
            }
          }
        }
      }
    }
    
    return result;
  }  
  
  /**
   * Bestimmt alle Straenabschnitte, die durch eine Menge von MQs gegeben sind
   * 
   * @param connection DAV-Verbindung
   * @param mq Sammlung von MQ, die die zu bestimmenden Straenabschnitte vorgeben
   * @return Sammlung aller gefundenen Straenabschnitte
   */
  private Map<String, StrassenAbschnitt> bestimmeStrassenAbschnitte(ClientDavInterface connection,
      Map<String, MessQuerschnitt> mq)
  {
    Map<String, StrassenAbschnitt> result = new HashMap<String, StrassenAbschnitt> ();
    
    SystemObjectType type = connection.getDataModel().getType("typ.straenAbschnitt");
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.straenAbschnitt");
    
    for (SystemObject obj: type.getElements())
    {
      if (obj.isValid())
      {
        Data data = obj.getConfigurationData(atg);
        if (data != null)
        {
          SystemObject von = data.getItem("vonMessQuerschnitt").asReferenceValue().getSystemObject();
          MessQuerschnitt vonMq = mq.get(von.getPid());
          if (vonMq != null)
          {  
            SystemObject bis = data.getItem("bisMessQuerschnitt").asReferenceValue().getSystemObject();
            MessQuerschnitt bisMq = mq.get(bis.getPid());
            if (bisMq != null)
            {
              result.put(obj.getPid(), new StrassenAbschnitt(obj, vonMq, bisMq));
            }
          }  
        }
      }
    }
    
    return result;
  }
  
}
