/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.konfiguration;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein Strassensegment modelliert.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:24 $ / ($Author: Sans $)
 */
public class StrassenSegment extends DavProxyObjekt 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Erstellt eine neue Instanz vom Typ <code>StrassenSegment</code>
   * 
   * @param davObjekt das zuzuordnente Dav-Objekt
   * @param laenge die Lnge in m des Strassensegments
   */
  protected StrassenSegment(SystemObject davObjekt, long laenge) 
      
  {
    super(davObjekt);
    _laenge = laenge;
  }
  
  /**
   * Lnge in m dieses Straensegments
   */
  private long _laenge;
  
  /**
   * @return die Lnge in m dieses Straensegments
   */
  public long getLaenge()
  {
    return _laenge; 
  }
  
  /**
   * Die Nachfolger-Strassensegmente zu diesem Strassensegment
   */
  protected Set<StrassenSegment> _nachfolger = new LinkedHashSet<StrassenSegment>();
  
  /**
   * Read-Only-Zugriff auf die Nachfolger-Strassensegmente zu diesem Strassensegment
   * 
   * @return Sammlung der Nachfolger-StrassenSegmente
   */
  public Set<StrassenSegment> getNachFolger()
  {
    return Collections.unmodifiableSet(_nachfolger);
  }
  
  /**
   * Fgt diesem Segment ein Nachfolger-Segment hinzu
   *  
   * @param segment das Nachfolger-Segment
   */
  void addNachfolger(StrassenSegment segment)
  {
    _nachfolger.add(segment);
  }
  
  /**
   * Die Vorgnger-Strassensegmente zu diesem Strassensegment
   */
  protected Set<StrassenSegment> _vorgaenger = new LinkedHashSet<StrassenSegment>();
  
  /**
   * Read-Only-Zugriff auf die Vorgnger-Strassensegmente zu diesem Strassensegment
   * 
   * @return Sammlung der Vorgnger-StrassenSegmente
   */
  public Set<StrassenSegment> getVorgaenger()
  {
    return Collections.unmodifiableSet(_vorgaenger);
  }
  
  /**
   * Fgt diesem Segment ein Vorgnger-Segment hinzu
   *  
   * @param segment das Vorgnger-Segment
   */
  void addVorgaenger(StrassenSegment segment)
  {
    _vorgaenger.add(segment);
  }
  
  /**
   * Die Nachfolger-Strassensegmente auf derselben Strae zu diesem Strassensegment
   */
  protected   Set<StrassenSegment> _nachfolgerAufStrasse = new LinkedHashSet<StrassenSegment>();
  
  /**
   * Read-Only-Zugriff auf die Nachfolger-Strassensegmente auf derselben Strae 
   * zu diesem Strassensegment
   * 
   * @return Sammlung der Nachfolger-StrassenSegmente auf derselben Strae
   */
  public Set<StrassenSegment> getNachfolgerAufStrasse()
  {
    return Collections.unmodifiableSet(_nachfolgerAufStrasse);
  }
  
  /**
   * Fgt diesem Segment ein Nachfolger-Segment auf derselben Strae hinzu
   *  
   * @param segment das Nachfolger-Segment auf derselben Strae
   */
  void addNachfolgerAufStrasse(StrassenSegment segment)
  {
    _nachfolgerAufStrasse.add(segment);
  }
  
  /**
   * Die Vorgnger-Strassensegmente auf derselben Strae zu diesem Strassensegment
   */
  protected Set<StrassenSegment> _vorgaengerAufStrasse = new LinkedHashSet<StrassenSegment>();
  
  /**
   * Read-Only-Zugriff auf die Vorgnger-Strassensegmente auf derselben Strae
   * zu diesem Strassensegment
   * 
   * @return Sammlung der Vorgnger-StrassenSegmente auf derselben Strae
   */
  public Set<StrassenSegment> getVorgaengerAufStrasse()
  {
    return Collections.unmodifiableSet(_vorgaengerAufStrasse);
  }
  
  /**
   * Fgt diesem Segment ein Vorgnger-Segment auf derselben Strae hinzu
   *  
   * @param segment das Vorgnger-Segment auf derselben Strae
   */
  void addVorgaengerAufStrasse(StrassenSegment segment)
  {
    _vorgaengerAufStrasse.add(segment);
  }
  
  /**
   * @return das auf der Strasse vorhergehende Strassensegment mit der kleinsten Lnge
   * oder null, falls kein Vorgnger auf Strasse existiert - wennn es mehrere Segmente
   * mit derselben Lnge gibt, wird das erste zurckgegeben
   */
  public StrassenSegment getKuerzestenVorgaengerAufStrasse()
  {
    StrassenSegment result = null;
    for (StrassenSegment segment: getVorgaengerAufStrasse())
    {
      if (result == null)
      {
        result = segment;
      }
      else
      {
        if (segment.getLaenge() < result.getLaenge())
        {
          result = segment;
        }
      }
    }
    return result;
  }
  
  /**
   * @return das auf der Strasse nachfolgende Strassensegment mit der kleinsten Lnge
   * oder null, falls kein Nachfolger auf Strasse existiert - wennn es mehrere Segmente
   * mit derselben Lnge gibt, wird das erste zurckgegeben
   */
  public StrassenSegment getKuerzestenNachfolgerAufStrasse()
  {
    StrassenSegment result = null;
    for (StrassenSegment segment: getNachfolgerAufStrasse())
    {
      if (result == null)
      {
        result = segment;
      }
      else
      {
        if (segment.getLaenge() < result.getLaenge())
        {
          result = segment;
        }
      }
    }
    return result;
  }
  
  /**
   * Methode prft bei ueren Straensegmenten ob das Straensegment in einem Autobahnkreuz oder Autobahndreieck beginnt.
   * Bei inneren Straensegmenten wird geprft ob der Straenknoten in dem das Straensegment liegt ein Autobahnkreuz 
   * oder Autobahndreieck ist.
   * (Methode wird in den abgeleiteten Klassen berschrieben)
   * 
   * @return Autobahnkreuz, Autobahndreick: true, sonst false
   */
  public boolean beginntInAutobahnKreuzOderDreieck ()
  {
    return false;
  }

  /**
   * Methode prft bei ueren Straensegmenten ob das Straensegment in einem Autobahnkreuz oder Autobahndreieck endet.
   * Bei inneren Straensegmenten wird geprft ob der Straenknoten in dem das Straensegment liegt ein Autobahnkreuz 
   * oder Autobahndreieck ist.
   * (Methode wird in den abgeleiteten Klassen berschrieben)
   * 
   * @return Autobahnkreuz, Autobahndreick: true, sonst false
   */
  public boolean endetInAutobahnKreuzOderDreieck ()
  {
    return false;
  }

  /**
   * Methode prft bei ueren Straensegmenten ob das Segment am Autobahnanfang beginnt. 
   * Bei inneren Straensegmenten wird geprft, ob der Straenknoten in dem das 
   * Straensegment liegt ein Autobahnanfang ist.
   * (Methode wird in den abgeleiteten Klassen berschrieben)
   * 
   * @return Autobahnanfang: true, sonst false
   */
  public boolean beginntAmAutobahnAnfang ()
  {
    return false;  
  }
  
  /**
   * Methode prft bei ueren Straensegmenten ob das Segment am Autobahnende endet. 
   * Bei inneren Straensegmenten wird geprft, ob der Straenknoten in dem das 
   * Straensegment liegt ein Autobahnende ist.
   * (Methode wird in den abgeleiteten Klassen berschrieben)
   * 
   * @return Autobahnende: true, sonst false
   */
  public boolean endetAmAutobahnEnde ()
  {
    return false;  
  }
  
  /**
   * Comparator, der eine Reihenfolge von <code>MessQuerschnitt</code>-
   * Instanzen nach dem Wert des Attributs Offset vorgibt
   * 
   * @author Dambach Werke GmbH
   */
  static class MqComparator implements Comparator<MessQuerschnitt>
  {
    /*
     * (Kein Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(MessQuerschnitt mq1, MessQuerschnitt mq2)
    {
      if (mq1.getOffset() < mq2.getOffset())
      {
        return -1;
      }
      else if (mq1.getOffset() > mq2.getOffset()) 
      {
        return 1;
      }
      else
      {
        return 0;
      }
    } 
  }  
  
  /**
   * Die Instanz von <code>MqComparator</code>, die fr alle Strassensegmente benutzt wird
   */
  static private MqComparator _mqComparator = new MqComparator();
     
  /**
   * Die Messquerschnitte, die auf diesem Strassensegment liegen
   */  
  SortedSet<MessQuerschnitt> _messQuerschnitte = new TreeSet<MessQuerschnitt>(_mqComparator);
  
  /**
   * Read only Zugriff auf die Messquerschnitte, die auf diesem 
   * Strassensegment liegen
   * 
   * @return die Messquerschnitte, die auf diesem Strassensegment liegen
   * - sortiert aufsteigend nach Offset
   */ 
  public SortedSet<MessQuerschnitt> getMessQuerschnitte()
  {
    return Collections.unmodifiableSortedSet(_messQuerschnitte);
  }
  
  /**
   * Fgt den Messquerschnitten auf diesem Strassensegment einen hinzu
   * 
   * @param mq der hinzuzufgende Messquerschnitt
   */
  void addMessQuerschnitt(MessQuerschnitt mq)
  {
    _messQuerschnitte.add(mq);
  }
  
  /**
   * Die Strassenteilsegmente dieses Strassensegments
   */  
  List<StrassenTeilSegment> _strassenTeilSegmente = new LinkedList<StrassenTeilSegment>();
  
  /**
   * Read only Zugriff auf die Strassenteilsegmente dieses Strassensegments
   * 
   * @return die Strassenteilsegmente dieses Strassensegments
   */ 
  public List<StrassenTeilSegment> getStrassenTeilSegmente()
  {
    return Collections.unmodifiableList(_strassenTeilSegmente);
  }
  
  /**
   * Fgt den Strassenteilsegmenten dieses Strassensegments eines hinzu
   * 
   * @param strassenTeilSegment das neue Strassenteilsegment
   */
  void addStrassenTeilSegment(StrassenTeilSegment strassenTeilSegment)
  {
    _strassenTeilSegmente.add(strassenTeilSegment);
  }
  
  
}
