/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.meldungserstellung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.allgemein.RDSEreignis;
import de.bsvrz.ste.rdstmc.konfiguration.StrassenSegment;
import de.bsvrz.ste.rdstmc.parameter.RDSParameterDaten;
import de.bsvrz.ste.rdstmc.situation.Situation;
import de.bsvrz.ste.rdstmc.situation.SituationEigenschaften;
import de.bsvrz.ste.rdstmc.verkehrsanalyse.StoerfallSituation;
import de.bsvrz.ste.rdstmc.verkehrsanalyse.StoerfallSituationDaten;
import de.bsvrz.ste.rdstmc.verkehrsanalyse.StoerfallSituationListener;
import de.bsvrz.sys.funclib.debug.Debug;

public class Route implements StoerfallSituationListener

{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Comparator, der eine Reihenfolge von <code>EreignisDaten</code>
   * nach den Werten der Attribute offset und start vorgibt
   * 
   * @author Dambach Werke GmbH
   */
  static class EreignisDatenComparator implements
    Comparator<EreignisDaten>
  {
    /*
     * (Kein Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(EreignisDaten d1, EreignisDaten d2)
    {
      if (d1.startSegmentOffset + d1.start < d2.startSegmentOffset + d2.start)
      {
        return -1;
      }
      else if (d1.startSegmentOffset + d1.start > d2.startSegmentOffset + d2.start) 
      {
        return 1;
      }
      else
      {
        return 0;
      }
    } 
  }
  
  /**
   * Instanz von <code>EreignisDatenComparator</code>, die fr alle
   * Strfallsituationen benutzt wird
   */ 
  static private EreignisDatenComparator EREIGNIS_DATEN_COMPARATOR = 
    new EreignisDatenComparator();
  
  
 
  private List<StrassenSegment> _segmente = 
    new LinkedList<StrassenSegment>();
  
  public List<StrassenSegment> getSegmente()
  {
    return Collections.unmodifiableList(_segmente);
  }
  
  Meldungserstellung _meldungserstellung;
  
  public RDSParameterDaten getParameter()
  {
    return _meldungserstellung.getParameter();
  }
  
  public Route(Meldungserstellung meldungserstellung, List<StrassenSegment> segmente)
  {
    _meldungserstellung = meldungserstellung;
    _segmente.addAll(segmente);
  }
     
  private Map<StrassenSegment, List<StoerfallSituationDaten>>_stoerfaelleAufSegmenten = new 
    LinkedHashMap<StrassenSegment, List<StoerfallSituationDaten>>();
  
  List<StoerfallSituationDaten> getStoerfalleAufSegment(StrassenSegment segment)
  {
    return _stoerfaelleAufSegmenten.get(segment);
  }
  
  public void neueStoerfallSituation(StoerfallSituation quelle, 
      List<StoerfallSituationDaten> daten)
  {
    if (getSegmente().contains(quelle.getSegment()))
    { 
      synchronized(_meldungserstellung.mutexEreignis)
      {
        _stoerfaelleAufSegmenten.put(quelle.getSegment(), daten);
        fireEreignisListener();
      }
    }
    else
    {
      throw new IllegalArgumentException("Straensegment " + quelle.getSegment() +
          " wurde an eine Route bergeben, die dieses Segment nicht enthlt");
    }  
  }
  
  // Bestimme Ereignisdaten fr die ganze Route
  private List<EreignisDaten> abbildung()
  {
    List<EreignisDaten> result = new ArrayList<EreignisDaten>();
    
    long offset = 0;
    
    for (StrassenSegment segment: getSegmente())
    {
      List<StoerfallSituationDaten> liste = _stoerfaelleAufSegmenten.get(segment);
      
      if (liste != null)
      {
        for (StoerfallSituationDaten daten: liste)
        {
          RDSEreignis ereignis = getParameter().getRDSAbbildungEreignis().get(daten.zustand);
          if (ereignis != null)
          {  
            result.add(new EreignisDaten(offset, offset, segment, segment,
                ereignis, daten.start, daten.ende, daten.lokal));
          }  
        }
      }
        
      offset += segment.getLaenge();
    }
    
    
//    for (StrassenSegment segment: _stoerfaelleAufSegmenten.keySet())
//    {  
//      // Bestimme Offset
//      long offset = 0;
//      for (StrassenSegment lauf: getSegmente())
//      {
//        if (lauf == segment)
//        {
//          break;
//        }
//        
//      } 
//      
//      // Bestimme Ereignisdaten fr die ganze Route
//      List<StoerfallSituationDaten> liste = _stoerfaelleAufSegmenten.get(segment);
//      
//      for (StoerfallSituationDaten daten: liste)
//      {
//        RDSEreignis ereignis = getParameter().getRDSAbbildungEreignis().get(daten.zustand);
//        if (ereignis != null)
//        {  
//          result.add(new EreignisDaten(offset, offset, segment, segment,
//              ereignis, daten.start, daten.ende, daten.lokal));
//        }  
//      }
//    }
//    Collections.sort(result, EREIGNIS_DATEN_COMPARATOR);
    return result;
  }      
    
  long getOffset(StrassenSegment segment)
  {
    long result = 0;
    for (StrassenSegment routenSegment: getSegmente())
    {
      if (routenSegment == segment)
      {
        return result;
      }
      else
      {
        result += routenSegment.getLaenge();
      }
    }
    throw new IllegalArgumentException("Segment " + segment.getDavObjekt().getPid() + 
        " in Route nicht vorhanden");
  }
  
  
  /**
   * Sammlung aller <code>EreignisDatenListener</code>
   */
  private Collection<EreignisListener> _ereignisListener = 
    new ArrayList<EreignisListener>();
  
  public void addEreignisListener(EreignisListener listener)
  {
    synchronized(_ereignisListener)
    {
    _ereignisListener.add(listener);
    }
  }
  
  public void renmoveEreignisListener(EreignisListener listener)
  {
    synchronized(_ereignisListener)
    {
    _ereignisListener.remove(listener);
    }
  }
  
  private List<EreignisDaten> _ereignisDaten = new LinkedList<EreignisDaten>();
  
  List<EreignisDaten> getEreignisDaten()
  {
    return Collections.unmodifiableList(_ereignisDaten);
  }
  
  
  /**
   * Benachrichtige alle angemeldeten Listener vom Typ <code>EreignisDatenDaten</code>
   */
  public void fireEreignisListener()
  {
    List<EreignisDaten> daten = abbildung();
    daten = laengenAbgleich(daten);
    daten = laengenBestimmung(daten);
    /* Stau-Plausibilitt: ALT Anfang */
    plausibilisiereDaten(daten);
    /* Stau-Plausibilitt: ALT Ende */
    daten = findeSchaerfsteZustaende(daten);
    
    _ereignisDaten.clear();
    _ereignisDaten.addAll(daten);
    
    synchronized(_ereignisListener)
    {
      for (EreignisListener listener: _ereignisListener)
      {
        listener.neueEreignisse(this, daten);
      }
    }  
  }
  
  private List<EreignisDaten> findeSchaerfsteZustaende(List<EreignisDaten> datenListe)
  {
    List<EreignisDaten> result = new LinkedList<EreignisDaten>();
    
    Map<StrassenSegment, RDSEreignis> schaerfsteZustaende =
      new LinkedHashMap<StrassenSegment, RDSEreignis>();
    
    for (EreignisDaten daten: datenListe)
    {
      StrassenSegment segment = daten.startSegment;
      while (segment != null)
      {
        RDSEreignis zustand = schaerfsteZustaende.get(segment);
        
        if (zustand == null || zustand.compareTo(daten.ereignis) < 0)
        {
          schaerfsteZustaende.put(segment, daten.ereignis);
        }
        if (segment != daten.endSegment)
        {
          segment = segment.getKuerzestenNachfolgerAufStrasse();
        }
        else
        {
          segment = null;
        }
      }
    }
    // schaerfsteZustaende enthlt jetzt fr jedes vorkommende Strassensegment den
    // schrfsten Zustand
        
    for (EreignisDaten daten: datenListe)
    {
      LinkedList<StrassenSegment> segmentListe = new LinkedList<StrassenSegment>(); 
      StrassenSegment segment = daten.startSegment;
      
      while (segment != null)
      {
        RDSEreignis zustand = schaerfsteZustaende.get(segment);
                
        if (daten.ereignis.compareTo(zustand) >= 0)
        {
          segmentListe.add(segment);
        }
        if (segment != daten.endSegment)
        {
          segment = segment.getKuerzestenNachfolgerAufStrasse();
        }
        else
        {
          segment = null;
        }
      }
      

      // segmentListe enthlt jetzt fr das aktuelle Datum die Straensegmente,
      // deren schrfster Zustand zum Datum passt - oder ist leer, leere Liste bedeutet
      // dasentsprechende Element wird vollstndig von einem anderen Element mit
      // schrferem Zustand dominiert und dementsprechend eliminiert.
      if (!segmentListe.isEmpty())
      {
        StrassenSegment startSegment = segmentListe.getFirst();
        StrassenSegment endSegment = segmentListe.getLast();
        if (startSegment == daten.startSegment && endSegment == daten.endSegment)
        {
          // Es hat sich fr dieses Element nichts gendert
          // Element wird unverndert bernommen
          result.add(daten);
        }
        else
        {
          // Fr dieses Element wurden Segmente weggenommen
          // Es muss ein entsprechendes neues Element erzeugt werden
          long startSegmentOffset;
          long start;
          long endSegmentOffset;
          long ende;
          if (startSegment == daten.startSegment)
          {
            // es wurden hinten Segmente weggenommen
            startSegmentOffset = daten.startSegmentOffset;
            start = daten.start;
            endSegmentOffset = getOffset(endSegment);
            ende = endSegment.getLaenge();
          }
          else if (endSegment == daten.endSegment)
          {
            // es wurden vorne Segmente weggenommen
            startSegmentOffset = getOffset(startSegment);
            start = 0;
            endSegmentOffset = daten.endSegmentOffset;
            ende = daten.ende;
          }
          else
          {
            // es wurden hinten und vorne Segmente weggenommen
            startSegmentOffset = getOffset(startSegment);
            start = 0;
            endSegmentOffset = getOffset(endSegment);
            ende = endSegment.getLaenge();
          }
          result.add(new EreignisDaten(startSegmentOffset, endSegmentOffset, startSegment,
              endSegment, daten.ereignis, start, ende, daten.lokal));
        }
      }  
    }
    
    /*
    String text = "ERGEBNIS SCHRFSTE ZUSTNDE";
    for (EreignisDaten dat: result)
    {
      text += "\n" + dat.ereignis + ": Lnge " + dat.getLaenge() + "m beginnend auf " + 
        dat.startSegment.getDavObjekt().getPid() + " bei " + dat.start + "m";
    }
    _debug.config(text);
    */
    return result;
  }
  
  private List<EreignisDaten> laengenAbgleich(List<EreignisDaten> ereignisDaten)
  {
    LinkedList<EreignisDaten> result = new LinkedList<EreignisDaten>();
        
    Map<RDSEreignis, List<EreignisDaten>> zustandsMap = new LinkedHashMap<RDSEreignis, List<EreignisDaten>>();
    Map<RDSEreignis, LinkedList<EreignisDaten>> resultMap = new LinkedHashMap<RDSEreignis, LinkedList<EreignisDaten>>();
    
    // Verteile die EreignisDaten auf RDS-Ereignis-bezogenen Listen und erzeuge ebensolche Ergebnislisten
    for(EreignisDaten daten:  ereignisDaten)
    {
      RDSEreignis zustand = daten.ereignis;
      List<EreignisDaten> liste = zustandsMap.get(zustand);
      if (liste == null)
      {
        liste = new LinkedList<EreignisDaten>();
        zustandsMap.put(zustand, liste);
        resultMap.put(zustand, new LinkedList<EreignisDaten>());
      }
      liste.add(daten);
    }
    
    for (RDSEreignis zustand: zustandsMap.keySet())
    {
      List<EreignisDaten> liste = zustandsMap.get(zustand);
      LinkedList<EreignisDaten> ergebnis = resultMap.get(zustand);
      
      for(EreignisDaten daten: liste)
      {
        if (ergebnis.isEmpty())
        {
          ergebnis.add(new EreignisDaten(daten));
        }
        else
        {
          EreignisDaten vorgaenger = ergebnis.getLast();
        
          long start = daten.startSegmentOffset + daten.start;
          long ende = vorgaenger.endSegmentOffset + vorgaenger.ende;

          if (start - ende  < getParameter().getRDSDMax())
          {
            EreignisDaten d = new EreignisDaten(vorgaenger.startSegmentOffset, daten.endSegmentOffset, vorgaenger.startSegment, 
                daten.endSegment, vorgaenger.ereignis, vorgaenger.start, daten.ende, vorgaenger.lokal == daten.lokal ? 
                    vorgaenger.lokal : null);
            ergebnis.removeLast();
            ergebnis.add(d);
          }
          else
          {
            ergebnis.add(new EreignisDaten(daten));
          }
        }
      }
    }
    
    for (List<EreignisDaten> liste: resultMap.values())
    {
      for(EreignisDaten daten: liste)
      {
        result.add(daten);
      }
    }
    
    Collections.sort(result, EREIGNIS_DATEN_COMPARATOR);
    
//    for(EreignisDaten daten:  ereignisDaten)
//    {
//      if (result.isEmpty())
//      {
//        result.add(new EreignisDaten(daten));
//      }
//      else
//      {
//        EreignisDaten vorgaenger = result.getLast();
//        
//        if (daten.ereignis.compareTo(vorgaenger.ereignis) == 0)
//        {
//          long start = daten.startSegmentOffset + daten.start;
//          long ende = vorgaenger.endSegmentOffset + vorgaenger.ende;
//
//          if (start - ende  < getParameter().getRDSDMax())
//          {
//            EreignisDaten d = new EreignisDaten(vorgaenger.startSegmentOffset, daten.endSegmentOffset, vorgaenger.startSegment, 
//                daten.endSegment, vorgaenger.ereignis, vorgaenger.start, daten.ende, vorgaenger.lokal == daten.lokal ? 
//                    vorgaenger.lokal : null);
//            result.removeLast();
//            result.add(d);
//          }
//          else
//          {
//            result.add(new EreignisDaten(daten));
//          }
//        }  
//        else
//        {
//          result.add(new EreignisDaten(daten));
//        }
//      }
//    }
    
    return result;
  }
  
  private List<EreignisDaten> laengenBestimmung(List<EreignisDaten> daten)
  {
    List<EreignisDaten> result = new LinkedList<EreignisDaten>();
    int dStoerfallAnfang = getParameter().getRDSDStoerfallAnfang();
    int dStoerfallEnde = getParameter().getRDSDStoerfallEnde();
    int mindestLaenge = getParameter().getRDSMindestLaenge();
    
    for (EreignisDaten lauf: daten)
    {
      /*
       * RDS-Ereignis hat Richtung entgegen der Fahrtrichtung - 
       * deshalb wird vor mit dStoerFallEnde (Parameter RDSStoerfallEnde) 
       * und nach mit dStoerFallAnfang (Parameter RDSStoerfallAnfang)
       * berechnet
       */
      long vor = dStoerfallEnde > lauf.start ? lauf.start : dStoerfallEnde;
      long nach = dStoerfallAnfang > lauf.endSegment.getLaenge() - lauf.ende ? 
          lauf.endSegment.getLaenge() - lauf.ende : dStoerfallAnfang;
      
      if (lauf.endSegmentOffset + lauf.ende + nach - 
          (lauf.startSegmentOffset + lauf.start - vor) >= mindestLaenge)
      {
        result.add(new EreignisDaten(lauf.startSegmentOffset, lauf.endSegmentOffset, lauf.startSegment, 
            lauf.endSegment, lauf.ereignis, lauf.start - vor, lauf.ende + nach, lauf.lokal));
      }
    }
    
    return result;
  }
  
  /* Stau-Plausibilitt: ALT Anfang */
  private void plausibilisiereDaten(List<EreignisDaten> daten)
  {
    if (getParameter().getRDSPlausibStauverlaufsanalyse())
    {
      Iterator<EreignisDaten> it = daten.listIterator();
      
      while (it.hasNext())
      {
        EreignisDaten lauf = it.next();
        if (lauf.ereignis.compareTo(RDSEreignis.STAU) == 0)
        {  
          int startIndex = getSegmente().indexOf(lauf.startSegment);
          int endIndex = getSegmente().indexOf(lauf.endSegment);
          if (!_meldungserstellung.getVerkehrsanalyse().existiertStau(lauf.startSegment, lauf.start, 
              lauf.endSegment, lauf.ende))
          {
            it.remove();
          }
        }  
      }
    }
  }
  /* Stau-Plausibilitt: ALT Ende */
  
  SystemObject sucheBaustelle(StrassenSegment wurzelSegment, long offset)
  {
    if (getParameter().getRDSBeruecksichtigeBaustellen())
    {
      long bereichNach = getParameter().getRDSDBaustelleSuchbereichNach();
      
      long bereich = wurzelSegment.getLaenge() - offset;
      StrassenSegment segment = wurzelSegment;
      StrassenSegment endSegment = segment;
      long endOffset;
                  
      while (segment != null && bereich < bereichNach)
      {
        segment = segment.getKuerzestenNachfolgerAufStrasse();
        if (segment != null)
        {
          bereich += segment.getLaenge();
        }
        else
        {
          endSegment = segment;
        }
      }
      
    }
    
    return null;
  }
  
//  private SystemObject findeBaustelleNachEreignis(StrassenSegment startSegment, long startOffset,
//      StrassenSegment endSegment, long endOffset)
//  {
//    SystemObject result = null;
//    StrassenSegment segment = startSegment;
//    while (segment != null)
//    {
//      for (Situation baustelle: _meldungserstellung.getBaustellenSegmente().keySet())
//      {
//        SituationEigenschaften eigenschaften  = _meldungserstellung.getBaustellenSegmente().get(baustelle);
//        for (int i = 0; result == null && i < eigenschaften.strassenSegment.length; i += 1)
//        {
//          if (eigenschaften.strassenSegment[i].equals(segment.getDavObjekt()))
//          {
//            // Gemeinsames Segment gefunden
//            if (i == 1 && segment == endSegment) // Erstes Baustellensegment ist letztes Suchsegment
//            {
//              if (eigenschaften.startOffset <= endOffset)
//              {
//                result = eigenschaften.strassenSegment[i];
//              }
//            }
//            else if (i == eigenschaften.strassenSegment.length - 1 && segment == startSegment) // letztes Baustellensegment ist erstes Suchsegment
//            {
//              if (eigenschaften.endOffset >= startOffset)
//              {
//                result = eigenschaften.strassenSegment[i];
//              }
//            }
//            else
//            {
//              result = eigenschaften.strassenSegment[i];
//            }
//          }
//        }
//      }
//      
//      segment = segment != endSegment ? segment.getKuerzestenNachfolgerAufStrasse() : null;
//    }
//    return result;
//  }
//   
//  private SystemObject findeBaustelleVorEreignis(StrassenSegment startSegment, long startOffset,
//      StrassenSegment endSegment, long endOffset)
//  {
//    SystemObject result = null;
//    StrassenSegment segment = endSegment;
//    while (segment != null)
//    {
//      for (Situation baustelle: _meldungserstellung.getBaustellenSegmente().keySet())
//      {
//        SituationEigenschaften eigenschaften  = _meldungserstellung.getBaustellenSegmente().get(baustelle);
//        for (int i = eigenschaften.strassenSegment.length - 1; i >= 0; i -= 1)
//        {
//          if (eigenschaften.strassenSegment[i].equals(segment.getDavObjekt()))
//          {
//             // Gemeinsames Segment gefunden
//            if (i == eigenschaften.strassenSegment.length - 1 && segment == startSegment) // letztes Baustellensegment ist erstes Suchsegment
//            {
//              if (eigenschaften.endOffset >= startOffset)
//              {
//                result = eigenschaften.strassenSegment[i];
//              }
//            }
//            else if (i == 1 && segment == endSegment) // Erstes Baustellensegment ist letztes Suchsegment
//            {
//              if (eigenschaften.startOffset <= endOffset)
//              {
//                result = eigenschaften.strassenSegment[i];
//              }
//            }
//            else
//            {
//              result = eigenschaften.strassenSegment[i];
//            }
//          }
//        }
//      }
//      
//      segment = segment != startSegment ? segment.getKuerzestenVorgaengerAufStrasse() : null;
//    }
//    return result;
//  }
   
}
