/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.parameter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.allgemein.Beschleunigung;
import de.bsvrz.ste.rdstmc.allgemein.RDSEreignis;
import de.bsvrz.ste.rdstmc.allgemein.StoerfallZustand;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Daten der paramatrierenden Attributgruppe RDSModell
 *
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:26 $ / ($Author: Sans $)
 */
public class RDSParameterDaten 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   *  Erzeugt eine neue Instanz von <code>RDSModellDaten</code> aus einem 
   * DAV-Datensatz der ATG RDSModell
   * 
   * @param DAV-Datensatz der ATG RDSModell
   */
  RDSParameterDaten(Data datenSatz)
  {
    /*
     * RDSLSIAusdehnung
     */
    _rdsLSIAusdehnung = 
      datenSatz.getItem("RDSLSIAusdehnung").asUnscaledValue().longValue();
    
    /*
     * RDSSIVerfahren
     */
    ReferenceArray array = datenSatz.getReferenceArray("RDSSIVerfahren");
    if (array != null)
    {
      for (int i = 0; i < array.getLength(); i+=1)
      {
        _rdsSIVerfahren.add((Aspect)array.getReferenceValue(i).getSystemObject());
      }
    }
    
    /*
     * RDSVerzgerungsmatrix
     */
    Array verzMatrix = datenSatz.getArray("RDSVerzgerungsmatrix");
    if (verzMatrix != null)
    {  
      for (int i = 0; i < verzMatrix.getLength(); i += 1)
      {
        Data verz = verzMatrix.getItem(i);
        if (verz != null)
        {
          int x = verz.getItem("SituationVon").asUnscaledValue().intValue();
          int y = verz.getItem("SituationNach").asUnscaledValue().intValue();
          long wert = verz.getItem("Verzgerungswert").asUnscaledValue().longValue();

          _rdsVerzoegerungsMatrix[x][y] = wert;
        }
      }
    }
    
    /*
     * RDSPriorittenmatrix
     */
    Data prioMatrix = datenSatz.getItem("RDSPriorittenmatrix");
    if (prioMatrix != null)
    {
      for (int i = 0; i < rdsPrioritaetenMatrixZustaende.length; i += 1)
      {
        String zustand = rdsPrioritaetenMatrixZustaende[i];
        if (zustand != null)
        {
          Data zeile = prioMatrix.getItem(zustand);
          if (zeile != null)
          {
            for (int j = 0; j < rdsPrioritaetenMatrixGueteBereiche.length; j += 1)
            {
              String guete = rdsPrioritaetenMatrixGueteBereiche[j];
              if (guete != null)
              {
                int prio = zeile.getItem(guete).asUnscaledValue().intValue();
                _rdsPrioritaetenMatrix[i][j] = prio;
              }
            }
          }
        }
      }
    }
    
    /*
     * RDSBeruecksichtigeStauverlaufsanalyse
     */
    String stauVerlauf = 
      datenSatz.getItem("RDSBercksichtigeStauverlaufsanalyse").asTextValue().getText();
    _rdsBeruecksichtigeStauverlaufsanalyse = stauVerlauf != null && stauVerlauf.equals("Ja");
    
    /*
     * RDSPlausibStauverlaufsanalyse
     */
    String stauPlausibel = 
      datenSatz.getItem("RDSPlausibStauverlaufsanalyse").asTextValue().getText();
    _rdsPlausibStauverlaufsanalyse = stauPlausibel != null && stauPlausibel.equals("Ja");
       
    /*
     * RDSNichtAddieren
     */
    String nichtAddieren = 
      datenSatz.getItem("RDSNichtAddieren").asTextValue().getText();
    _rdsNichtAddieren = nichtAddieren != null && nichtAddieren.equals("Ja");
    
    /*
     * RDSDMax
     */
    _rdsDMax = datenSatz.getItem("RDSDMax").asUnscaledValue().intValue();
    
    /*
     * RDSDStoerfallAnfang
     */
    _rdsDStoerfallAnfang = datenSatz.getItem("RDSDStoerfallAnfang").asUnscaledValue().intValue();
    
    /*
     * RDSDStoerfallEnde
     */
    _rdsDStoerfallEnde = datenSatz.getItem("RDSDStoerfallEnde").asUnscaledValue().intValue();
   
    /*
     * RDSMindestLaenge
     */
    _rdsMindestLaenge = datenSatz.getItem("RDSMindestLaenge").asUnscaledValue().intValue();
    
    /*
     * RDSDMaxEinzelStoerfall
     */
    _rdsDMaxEinzelStoerfall = datenSatz.getItem("RDSDMaxEinzelStoerfall").asUnscaledValue().intValue();
    
    /*
     * RDSBaustelleSuchbereichVor
     */
    _rdsBaustelleSuchbereichVor = 
      datenSatz.getItem("RDSBaustelleSuchbereichVor").asUnscaledValue().intValue();    
    
    /*
     * RDSBaustelleSuchbereichNach
     */
    _rdsBaustelleSuchbereichNach = 
      datenSatz.getItem("RDSBaustelleSuchbereichNach").asUnscaledValue().intValue();
    
    /*
     * RDSBercksichtigeBaustellen
     */
    String beruecksichtigeBaustellen = 
      datenSatz.getItem("RDSBercksichtigeBaustellen").asTextValue().getText();
    _rdsBeruecksichtigeBaustellen = beruecksichtigeBaustellen != null && 
    beruecksichtigeBaustellen.equals("Ja");
  
    /*
     * RDSAbbildungsmatrix
     */
    Array abbildungsMatrix = datenSatz.getArray("RDSAbbildungsmatrix");
    if (abbildungsMatrix != null)
    {  
      for (int i = 0; i < abbildungsMatrix.getLength(); i += 1)
      {
        Data abbildung = abbildungsMatrix.getItem(i);
        if (abbildung != null)
        {
          int idZustand = abbildung.getItem("StrfallSituation").asUnscaledValue().intValue();
          StoerfallZustand zustand = StoerfallZustand.get(idZustand);
          if (zustand == null)
          {
            _debug.error("Fr Zustands-ID " + idZustand + " konnte kein Stoerfallzustand besimmt werden");
            continue;
          }
          int idEreignis = abbildung.getItem("RDSEreignisBeschreibung").asUnscaledValue().intValue();
          RDSEreignis ereignis = RDSEreignis.get(idEreignis);
          if (zustand == null)
          {
            _debug.error("Fr RDS-Eriegnis-ID " + idEreignis + " konnte kein RDS-Ereignis besimmt werden");
            continue;
          }
          _rdsAbbildungEreignis.put(zustand, ereignis);
                   
          Array codes = abbildung.getArray("RDSEreignisCode");
          if (codes != null)
          {  
            Collection<String> codeList = new ArrayList<String>();
          
            for (int j = 0; j < codes.getLength(); j += 1)
            {
              String code = "" + codes.getItem(j).asUnscaledValue().intValue();
              codeList.add(code);
             }
            _rdsAbbildungCode.put(ereignis, codeList);
          }
        }  
      }
      
      /*
       * RDSAusdehnungProLSI
       */
      Array RDSAusdehnungProLSI = datenSatz.getArray("RDSAusdehnungProLSI");
      if (RDSAusdehnungProLSI != null)
      {  
        for (int i = 0; i < RDSAusdehnungProLSI.getLength(); i += 1)
        {
          Data data = RDSAusdehnungProLSI.getItem(i);
          if (data != null)
          {
            SystemObject lsi = data.getItem("LSI").asReferenceValue().getSystemObject();
            long ausdehnungVor = data.getItem("AusdehnungVor").asUnscaledValue().longValue();
            long ausdehnungNach = data.getItem("AusdehnungNach").asUnscaledValue().longValue();

            _rdsAusdehnungProLSIVor.put(lsi, new Long(ausdehnungVor));
            _rdsAusdehnungProLSINach.put(lsi, new Long(ausdehnungNach));
          }  
        }
      }  
    }
  }
  
  /**
   * Wert des Parameters RDSSIVerfahren
   */
  private Collection<Aspect> _rdsSIVerfahren = new HashSet<Aspect>();
  
  /**
   * Bestimmt den Wert des Parameters RDSSIVerfahren
   * @return eine Sammlung der aktuell parametrierten Verfahren
   */
  public Collection<Aspect> getRDSSIVerfahren()
  {
    return _rdsSIVerfahren;
  }
  
  /**
   * Bestimmt einen Wert aus der Matrix RDSVerzgerungsmatrix
   * 
   * @param von der Ist-Zustand
   * @param nach der Soll-Zustand
   * @return Verzgerungszeit in s fr den bergange vom Ist-Zustand in den Sollzustand
   */
  public long getRDSVerzoegerungsMatrix(StoerfallZustand von, StoerfallZustand nach)
  { 
    return _rdsVerzoegerungsMatrix[von.ordinal()][nach.ordinal()] / Beschleunigung.WERT;
  }
  
  /**
   * Die Werte der RDSVerzgerungsmatrix
   */
  private long[][] _rdsVerzoegerungsMatrix = 
    new long[StoerfallZustand.values().length][StoerfallZustand.values().length];
  
  /**
   * Wert des Parameters RDSLSIAusdehnung
   */
  private long _rdsLSIAusdehnung = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSLSIAusdehnung
   * 
   * @return die Ausdehnung n m
   */
  public long getRDSLSIAusdehnung()
  {
    return _rdsLSIAusdehnung;
  }
  
  /**
   * Die Zustnde der RDSPriorttensmatrix
   */
  protected final static String[] rdsPrioritaetenMatrixZustaende = new String[StoerfallZustand.values().length];
  static
  {
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.STAU.ordinal()] = "Stau";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.STOCKENDER_VERKEHR.ordinal()] = "stockenderVerkehr";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.ZAEHFLIESSENDER_VERKEHR.ordinal()] = "zhflieenderVerkehr";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.DICHTER_VERKEHR.ordinal()] = "dichterVerkehr";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.LEBHAFTER_VERKEHR.ordinal()] = "lebhafterVerkehr";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.FREIER_VERKEHR.ordinal()] = "freierVerkehr";
    rdsPrioritaetenMatrixZustaende[StoerfallZustand.KEINE_AUSSAGE.ordinal()] = "keineAussage";
  }; 
    
  /**
   * Die Gte-Bereiche der RDSPriorttensmatrix
   */
  protected final static String[] rdsPrioritaetenMatrixGueteBereiche =
  { "Gte1bis10",
    "Gte11bis20",
    "Gte21bis30",
    "Gte31bis40",
    "Gte41bis50",
    "Gte51bis60",
    "Gte61bis70",
    "Gte71bis80",
    "Gte81bis90",
    "Gte91bis100" };
  
  /**
   * Die Werte der RDSPriorttensmatrix
   */
  private int[][] _rdsPrioritaetenMatrix = 
    new int[rdsPrioritaetenMatrixZustaende.length][rdsPrioritaetenMatrixGueteBereiche.length];
  
  /**
   * Bestimmt einen Wert aus der Matrix RDSPriorittenmatrix
   * 
   * @param zustand Strfall-Zustand
   * @param guete Gte fr die Messung dieses Zustands
   * @return Prioritt fr dieses Zustand/Gte-Paar
   */
  public int getRdsPrioritaetenMatrix(StoerfallZustand zustand, float guete)
  {
    int x = zustand.ordinal();
    int y = (int) (guete * 100 + 9) / 10 - 1;
    
    return _rdsPrioritaetenMatrix[x][y];
  }
  
  /**
   * Wert des Parameters RDSPlausibStauverlaufsanalyse
   */
  private boolean _rdsPlausibStauverlaufsanalyse;
  
  /**
   * Bestimmt den Wert des Parameters RDSPlausibStauverlaufsanalyse
   * 
   * @return true, wenn der Paramterwert "Ja" ist, false sonst
   */
  public boolean getRDSPlausibStauverlaufsanalyse()
  {
    return _rdsPlausibStauverlaufsanalyse;
  }
  
  /**
   * Wert des Parameters RDSBeruecksichtigeStauverlaufsanalyse
   */
  private boolean _rdsBeruecksichtigeStauverlaufsanalyse;
  
  /**
   * Bestimmt den Wert des Parameters RDSBeruecksichtigeStauverlaufsanalyse
   * 
   * @return true, wenn der Paramterwert "Ja" ist, false sonst
   */
  public boolean getRDSBeruecksichtigeStauverlaufsanalyse()
  {
    return _rdsBeruecksichtigeStauverlaufsanalyse;
  }
  
  /**
   * Wert des Parameters RDSNichtAddieren
   */
  private boolean _rdsNichtAddieren;
  
  /**
   * Bestimmt den Wert des Parameters RDSNichtAddieren
   * 
   * @return true, wenn der Paramterwert "Ja" ist, false sonst
   */
  public boolean getRDSNichtAddieren()
  {
    return _rdsNichtAddieren;
  }
  
  /**
   * Wert des Parameters RDSDMax
   */
  private int _rdsDMax = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDMax
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDMax()
  {
    return _rdsDMax;
  }
  
  /**
   * Wert des Parameters RDSDStoerfallAnfang
   */
  private int _rdsDStoerfallAnfang = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDStoerfallAnfang
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDStoerfallAnfang()
  {
    return _rdsDStoerfallAnfang;
  }
  
  /**
   * Wert des Parameters RDSDStoerfallEnde
   */
  private int _rdsDStoerfallEnde = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDStoerfallEnde
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDStoerfallEnde()
  {
    return _rdsDStoerfallEnde;
  }
  
  /**
   * Wert des Parameters RDSDMindestLaenge
   */
  private int _rdsMindestLaenge = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDMindestLaenge
   * 
   * @return den Parameterwert n m
   */
  public int getRDSMindestLaenge()
  {
    return _rdsMindestLaenge;
  }
  
  /**
   * Wert des Parameters RDSDMaxEinzelStoerfall
   */
  private int _rdsDMaxEinzelStoerfall = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDMaxEinzelStoerfall
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDMaxEinzelStoerfall()
  {
    return _rdsDMaxEinzelStoerfall;
  }
  
  /**
   * Wert des Parameters RDSDBaustelleSuchbereichVor
   */
  private int _rdsBaustelleSuchbereichVor = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDBaustelleSuchbereichVor
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDBaustelleSuchbereichVor()
  {
    return _rdsBaustelleSuchbereichVor;
  }
  
  /**
   * Wert des Parameters RDSDBaustelleSuchbereichNach
   */
  private int _rdsBaustelleSuchbereichNach = 0;
  
  /**
   * Bestimmt den Wert des Parameters RDSDBaustelleSuchbereichNach
   * 
   * @return den Parameterwert n m
   */
  public int getRDSDBaustelleSuchbereichNach()
  {
    return _rdsBaustelleSuchbereichNach;
  }
  
  /**
   * Wert des Parameters RDSBercksichtigeBaustellen
   */
  private boolean _rdsBeruecksichtigeBaustellen;
  
  /**
   * Bestimmt den Wert des Parameters RDSBercksichtigeBaustellen
   * 
   * @return true, wenn der Paramterwert "Ja" ist, false sonst
   */
  public boolean getRDSBeruecksichtigeBaustellen()
  {
    return _rdsBeruecksichtigeBaustellen;
  }
  
  /**
   * Mapping der Strfall Zustnde auf RDS/TMC Ereignisse
   */
  private Map<StoerfallZustand, RDSEreignis> _rdsAbbildungEreignis = 
    new HashMap<StoerfallZustand, RDSEreignis>();
  
  /**
   * @return Read only Zugriff auf das Mapping der Strfall Zustnde auf 
   * RDS/TMC Ereignisse
   */
  public Map<StoerfallZustand, RDSEreignis> getRDSAbbildungEreignis()
  {
    return Collections.unmodifiableMap(_rdsAbbildungEreignis) ;
  }
  
  /**
   * Mapping der Strfall Zustnde auf eine Sammlung von TMC-Ereignis-Codes
   */
  private Map<RDSEreignis, Collection<String>> _rdsAbbildungCode = 
    new HashMap<RDSEreignis, Collection<String>>();
  
  /**
   * @return Read only Zugriff aud das Mapping der Strfall Zustnde 
   * auf eine Sammlung von TMC-Ereignis-Codes
   */
  public Map<RDSEreignis, Collection<String>> getRDSAbbildungCode()
  {
    return Collections.unmodifiableMap(_rdsAbbildungCode) ;
  }
  
  /**
   * Mapping lokaler Sttfallindikatoren, die fr ihre angenommene Ausdehnung individuell 
   * parametriert sind, auf die individuelle angenommene Ausdehnung vor der Strung
   */
  private Map<SystemObject, Long> _rdsAusdehnungProLSIVor = 
    new HashMap<SystemObject, Long>();
  
  /**
   * Mapping lokaler Sttfallindikatoren, die fr ihre angenommene Ausdehnung individuell 
   * parametriert sind, auf die individuelle angenommene Ausdehnung nach der Strung
   */
  private Map<SystemObject, Long> _rdsAusdehnungProLSINach = 
    new HashMap<SystemObject, Long>();
  
  public long getRDSLSIAusdehnungVor(SystemObject lsi)
  {
    Long result = _rdsAusdehnungProLSIVor.get(lsi);
    return result != null ? result.longValue() : _rdsLSIAusdehnung / 2;
  }
  
  public long getRDSLSIAusdehnungNach(SystemObject lsi)
  {
    Long result = _rdsAusdehnungProLSINach.get(lsi);
    return result != null ? result.longValue() : _rdsLSIAusdehnung / 2;
  }
  
  /**
   * Keine relevanten nderungen
   */
  static public final byte FLAG_LEVEL0 = 0x00;
  
  /**
   * nderungen relevant Routen
   */
  static public final byte FLAG_LEVEL1 = 0x01;
  
  /**
   * nderungen relevant fr Strfallsituationen
   */
  static public final byte FLAG_LEVEL2 = 0x02;
  
  /**
   * nderungen relevant fr Strflle
   */
  static public final byte FLAG_LEVEL3 = 0x04;
  
  /**
   * Bestimmt die Level Flags (=Level der nderungen) zwischen dieser Instanz 
   * und einem bergebenem Objekt
   * 
   * @param daten die <code>RDSModellDaten</code>, deren Level-Flags in Bezug auf diese
   * Instanz bestimmt werden soll.
   *  
   * @return die Level-Flags, die sich aus dem Vergleich zwischen dieser Instanz und den
   * bergebenen Daten ergeben haben
   */
  public byte getLevelFlags(RDSParameterDaten daten)
  {
    byte result = FLAG_LEVEL0;

    if (daten != null)
    {
      /*
       * RDSLSIAusdehnung
       */
      if (getRDSLSIAusdehnung() != daten.getRDSLSIAusdehnung())
      {
        result |= FLAG_LEVEL3;
      }
            
      /*
       * RDSSIVerfahren
       */
      if (!getRDSSIVerfahren().equals(daten.getRDSSIVerfahren()))
      {
        result |= FLAG_LEVEL3;
      }
            
      /*
       * RDSVerzgerungsmatrix
       */
      if (!Arrays.deepEquals(_rdsVerzoegerungsMatrix, daten._rdsVerzoegerungsMatrix))
      {
        // nderung nicht relevant
        result |= FLAG_LEVEL0;
      }
            
      /*
       * RDSPriorittenmatrix
       */
      if (!Arrays.deepEquals(_rdsPrioritaetenMatrix, daten._rdsPrioritaetenMatrix))
      {
        result |= FLAG_LEVEL3;
      }
            
      /*
       * RDSBeruecksichtigeStauverlaufsanalyse
       */
      if (getRDSBeruecksichtigeStauverlaufsanalyse() != daten.getRDSBeruecksichtigeStauverlaufsanalyse())
      {
        result |= FLAG_LEVEL2;
      }
      
      /*
       * RDSPlausibStauverlaufsanalyse
       */
      if (getRDSPlausibStauverlaufsanalyse() != daten.getRDSPlausibStauverlaufsanalyse())
      {
        /* Stau-Plausibilitt: ALT Anfang */
        result |= FLAG_LEVEL1;
        /* Stau-Plausibilitt: ALT Ende */
        
        /* Stau-Plausibilitt: NEU Anfang */
//        result |= FLAG_LEVEL2;
        /* Stau-Plausibilitt: NEU Ende */
        
      }
         
      /*
       * RDSNichtAddieren
       */
      if (getRDSNichtAddieren() != daten.getRDSNichtAddieren())
      {
        result |= FLAG_LEVEL1;
      }
      
      /*
       * RDSDMax
       */
      if (getRDSDMax() != daten.getRDSDMax())
      {
        result |= FLAG_LEVEL1;
      }
      
      /*
       * RDSDStoerfallAnfang
       */
      if (getRDSDStoerfallAnfang() != daten.getRDSDStoerfallAnfang())
      {
        result |= FLAG_LEVEL1;
      }
      
      /*
       * RDSDStoerfallEnde
       */
      if (getRDSDStoerfallEnde() != daten.getRDSDStoerfallEnde())
      {
        result |= FLAG_LEVEL1;
      }
     
      /*
       * RDSMindestLaenge
       */
      if (getRDSMindestLaenge() != daten.getRDSMindestLaenge())
      {
        result |= FLAG_LEVEL1;
      }
      
      /*
       * RDSDMaxEinzelStoerfall
       */
      if (getRDSDMaxEinzelStoerfall() != daten.getRDSDMaxEinzelStoerfall())
      {
        result |= FLAG_LEVEL1;
      }
            
      /*
       * RDSBaustelleSuchbereichVor
       */
      if (getRDSDBaustelleSuchbereichVor() != daten.getRDSDBaustelleSuchbereichVor())
      {
        result |= FLAG_LEVEL1;
      }    
      
      /*
       * RDSBaustelleSuchbereichNach
       */
      if (getRDSDBaustelleSuchbereichNach() != daten.getRDSDBaustelleSuchbereichNach())
      {
        result |= FLAG_LEVEL1;
      }
      
      /*
       * RDSBercksichtigeBaustellen
       */
      if (getRDSBeruecksichtigeBaustellen() != daten.getRDSBeruecksichtigeBaustellen())
      {
        result |= FLAG_LEVEL1;
      }
            
      /*
       * RDSAbbildungsmatrix
       */
      if (!_rdsAbbildungEreignis.equals(daten._rdsAbbildungEreignis) ||
          !_rdsAbbildungCode.equals(daten._rdsAbbildungCode))
      {  
        result |= FLAG_LEVEL1;
      }
      
      /*
       * rdsAusdehnungProLSI
       */
      if (!_rdsAusdehnungProLSIVor.equals(daten._rdsAusdehnungProLSIVor) ||
          !_rdsAusdehnungProLSINach.equals(daten._rdsAusdehnungProLSINach))
      {
        result |= FLAG_LEVEL3;
      }
    }
    else
    {
      result = (FLAG_LEVEL1 | FLAG_LEVEL2 | FLAG_LEVEL3);
    }
    
    return result;
  }

}
