/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */


package de.bsvrz.ste.rdstmc.parameter;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.sys.funclib.concurrent.UnboundedQueue;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * <code>RDSParameterVerwaltung</code> verwaltet die parametrierenden Daten
 * - Modell-Parameter - der Attributgruppe atg.rdsModell. Die Klasse ist 
 * fr genau ein Konfigurationsobjekt vom Typ Landesmeldestelle und eine 
 * DAV-Verbindung instanziierbar.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.2 $ / $Date: 2009/01/27 13:54:22 $ / ($Author: Sans $)
 */
public class RDSParameterVerwaltung extends UnboundedQueue<RDSParameterDaten>
  implements ClientReceiverInterface, Runnable
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Erzeugt eine neue Instanz von <code>RDSParameter</code> im Kontext einer DAV-Verbindung
   * und eines Konfigurationsobjekts vom Typ Landesmeldestelle
   * 
   * @param connection die  Datenveteiler-Verbindung
   * @param obj das (Konfigurations-)Objekt, dessen Parameter ausgewertet werden
   */
  public RDSParameterVerwaltung(ClientDavInterface connection, ConfigurationObject lms)
  {
    super();
    
    _connection = connection;
    _lms = lms;
    
    Thread thread = new Thread(this);
    thread.setName("RDS-Parameter-Updater");
    thread.start();
    _debug.config(thread + " gestartet");
    
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
    _dataDescription = new DataDescription(atg, asp);
    
    _connection.subscribeReceiver(this, _lms, _dataDescription,
       ReceiveOptions.normal(), ReceiverRole.receiver());
    _debug.finer(_dataDescription + " zum Empfang angemeldet fr " + _lms);
  }
    
  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;
  
  /**
   * Objekt - die LMS - von dem die Parameter abgefragt werden
   */
  private ConfigurationObject _lms;
  
  public ConfigurationObject getLms()
  {
    return _lms;
  }
  
  /**
   * Datenbeschreibung, auf die die Instanz als Empfnger angemeldet ist
   */
  protected final DataDescription _dataDescription;
  
  /**
   * Die aktuellen Modell-Parameter
   */
  private RDSParameterDaten _aktuelleDaten = null;
  
  /**
   * Gibt belegte Resourcen frei
   */
  void dispose()
  {
    // _parameterListener.clear();
    _connection.unsubscribeReceiver(this, _lms, _dataDescription);
    _debug.finer("Empfang abgemeldet von " + _dataDescription + " fr " + _lms);
  }
  
  /**
   * Wartet maximal <code>timeout</code>ms auf das Vorhandensein von Parameterdaten
   * 
   * @param timeout die Zeit, die maximal gewartet wird
   * @return true wenn Daten vorhanden sind, false sonst
   */
  public boolean wartenAufParameter(long timeout)
  {
    synchronized (this)
    {
      if (timeout > 0)
      {  
        _debug.config("Warte max. " + timeout + "ms auf Parameterdaten");
                  
        if (_aktuelleDaten == null)
        {
          try
          {
            wait(timeout);
          }
          catch(InterruptedException e)
          {
          }
        }
      }
      return _aktuelleDaten != null;
    }  
  }
  
   
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] resultData)
  {
    for (ResultData data : resultData)
    {
      Aspect asp = data.getDataDescription().getAspect();
      AttributeGroup atg = data.getDataDescription().getAttributeGroup();
      
      if (atg.equals(_dataDescription.getAttributeGroup()) &&
          asp.equals(_dataDescription.getAspect())) 
      {
        if (data.hasData())
        {
          _debug.fine("Update Daten " + atg.getPid() + "/" + asp.getPid()+ " fr " + _lms.getName());
          put (new RDSParameterDaten(data.getData()));
        }
        else
        {
          _debug.config("Update keine Daten " + atg.getPid() + "/" + asp.getPid()+ " fr " + 
          _lms.getName() + " - DataState: " + data.getDataState()); 
        }
      }
    }
  }
  
  /**
   * Zugriff auf die Modell-Parameter
   * 
   * @return die aktuellen Werte der Attributgruppe atg.rdsModell als Instanz von
   * <code>RDSModellDaten</code>
   */
  public RDSParameterDaten getAktuelleDaten()
  {
    return _aktuelleDaten;
  }
    
  /**
   * Sammlung aller <code>ParameterListener</code>
   */
  private Collection<RDSParameterListener> _parameterListener = new ArrayList<RDSParameterListener>();
  
  /**
   * Registriert (fgt hinzu) einen <code>ParameterListener</code> bei dieser Instanz
   * 
   * @param listener der hinzuzufgende <code>ParameterListener</code>
   */
  public void addParameterListener(RDSParameterListener listener)
  {
    synchronized(_parameterListener)
    {
      _parameterListener.add(listener);
    }  
  }
  
  /**
   * De-Registriert (entfernt) einen <code>ParameterListener</code> bei dieser Instanz
   * 
   * @param listener der zu entfernende <code>ParameterListener</code>
   */
  public void removeParameterListener(RDSParameterListener listener)
  {
    synchronized(_parameterListener)
    {
      _parameterListener.remove(listener);
    }  
  }
  
  /**
   * Informiert alle registrierten <code>ParameterListener</code> ber nderungen an den Parametern
   * 
   * @param daten die genderten Daten als Instanz von <code>RDSModellDaten</code>  
   */
  protected void fireParameterListener(RDSParameterDaten daten)
  {
    synchronized(_parameterListener)
    {
      for (RDSParameterListener listener: _parameterListener)
      {
        listener.neueParameter(this, daten);
      }
    }  
  }
  
  /*
   * (Kein Javadoc)
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    while (true)
    {  
      try
      {
        RDSParameterDaten daten = take();
        synchronized(this)
        {
          _aktuelleDaten = daten;
          notifyAll();
        }  
        fireParameterListener(_aktuelleDaten);
      }
      catch (InterruptedException e)
      {
        _debug.warning("Thread wurde unerwartet unterbrochen");
      }
    }
  }
  
}
