/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */


package de.bsvrz.ste.rdstmc.rdstmc;

import java.lang.Thread.UncaughtExceptionHandler;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.ste.rdstmc.konfiguration.Konfiguration;
import de.bsvrz.ste.rdstmc.meldungserstellung.LocationMethode;
import de.bsvrz.ste.rdstmc.meldungserstellung.MeldungenGlobal;
import de.bsvrz.ste.rdstmc.meldungserstellung.Meldungserstellung;
import de.bsvrz.ste.rdstmc.meldungserstellung.PublikationInitial;
import de.bsvrz.ste.rdstmc.parameter.RDSParameterVerwaltung;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Applikationsklasse der SWE RDS/TMC.<br>
 * Fr das bergebene Netz und die bergebene LMS wird eine Applikations-
 * Instanz der SWE RDS/TMC erzeugt und initialisiert.<br>
 * SWE-spezifische Parameter:<br>
 * -netz: Das Netz, dessen Infrastrukturobjekte (Strassensegmente, Messquerschnitte etc.)
 * betrachtet werden. Da ein Netz weitere Netze enthalten kann, erfolgt die Auswertung rekursiv.<br>
 * -lms: Die Landesmeldestelle, auf welche diese Instanz der SWE sich bezieht.
 * Das Konfigurationsobjekt der LMS enthlt insbesondere die zu verwendeten RDS-Modell-Paramenter.
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.3 $ / $Date: 2010/03/15 10:07:30 $ / ($Author: Sans $)
 */
public class RDSTMCApp implements StandardApplication, MeldungenGlobal
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private Debug _debug;

   /**
   * -netz von Kommandozeile
   */
  private String _netzPid;
  
  /**
   * -lms von Kommandozeile
   */
  private String _lmsPid;
  
  /**
   * -konfigBereich von Kommandozeile
   */
  private String _konfigBereichPid;
  
  /**
   * -locationMethode von Kommandozeile
   */
  private int _locationMethodeId;
  
  /**
   * -publikationInitial von Kommandozeile
   */
  private int _publikationInitialId;
  
  /**
   * -idOrganisation von Kommandozeile
   */
  private String _idOrganisation;
  
  /**
   * -idNummer von Kommandozeile
   */
  private String _idNummer;
  
  /**
   * -ablaufOffset von Kommandozeile
   */
  private int _ablaufOffset;
  
  
  private ConfigurationArea _konfigBereich;
  
  private LocationMethode _locationMethode;

  private PublikationInitial _publikationInitial;
  
  private ClientDavInterface _connection;
  
  private Meldungserstellung _meldungserstellung;
  
  /**
   * Timeout in ms fr Daten der Parametierung
   */
  public final static long TIMEOUT = 5000;
  
  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public RDSTMCApp()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }
  
  /**
   * Extrahiert das Konfigurationsobjekt tur bergebenenen PID und fhrt ggf.vom Typ typ.netz 
   * eine Typprfung durch.
   * @param pid die PID des Konfigurationsobjektes, das extrahiert werden soll
   * @param typ der Typ, den das Konfigutationsobjekt haben muss - wenn hier <code>null</code> bergeben
   * wird, unterbleibt eine Typprfung.
   * @return das gefundene Konfigurationsobjekt
   * @throws IllegalArgumentException, falls Konfigurationsobjekt nicht gefunden oder 
   * vom falschen Typ
   */
  private ConfigurationObject getKonfigObjekt(String pid, String typ)
  {
    ConfigurationObject obj = null;
    
    try
    {
      obj = (ConfigurationObject)_connection.getDataModel().getObject(pid);
    }
    catch(ClassCastException e)
    {
      throw new IllegalArgumentException("Die PID \"" + pid + 
      "\" bezeichnet kein KonfigurationsObjekt");
    }
    
    if (obj == null)
    {
      throw new IllegalArgumentException("Zur PID \"" + pid + 
      "\" wurde kein Konfigurationsobjekt gefunden ");
    }
  
    if (typ != null && !obj.isOfType(typ))
    {
      throw new IllegalArgumentException("Das Konfigurationsobjekt \"" + 
          obj.getPid() + "\" ist nicht vom Typ " + typ); 
    }
    
    return obj;
  }
  
  /**
   * Extrahiert - nach Prfung - den Konfigurationsbereich aus dem Attribut {@link #_konfigBereich}
   * @return den gefunden Konfigurationsbereich
   * @throws IllegalArgumentException, falls Konfigurationsbereich nicht gefunden
   */
  private ConfigurationArea getKonfigBereich(String pid)
  {
    ConfigurationArea ca = null;
    if (pid != null && !pid.equals(""))
    {  
      ca = _connection.getDataModel().getConfigurationArea(pid);
      if (ca == null)
      {
        throw new IllegalArgumentException("Zur PID \"" + pid + 
        "\" wurde kein Konfigurationsbereich gefunden ");
      }
      return ca;
    }
    else
    {
      DataModel dm = _connection.getDataModel();
      ConfigurationObject co = dm.getConfigurationAuthority();
      AttributeGroup atg = dm.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
      Data data = co.getConfigurationData(atg);
      if (data != null)
      {
        TextArray array = data.getTextArray("defaultBereich");
        if (array.getLength() > 0)
        {
          ca = (ConfigurationArea)dm.getObject(array.getText(0));
        }
      }
      if (ca == null)
      {
        throw new IllegalArgumentException("Es konnte kein Default-Konfigurationsbereich bestimmt werden");
      }
    }
    return ca;
  }
  
  private LocationMethode getLocationMethode(int id)
  {
    LocationMethode locationMethode = LocationMethode.get(id);
    if (locationMethode == null)
    {
      throw new IllegalArgumentException("LocationMethode \"" + id + "\" wird nicht untersttzt");
    }
    return locationMethode;
  }  
  
  private PublikationInitial getPublikationInitial(int id)
  {
    PublikationInitial publikationInitial = PublikationInitial.get(id);
    if (publikationInitial == null)
    {
      throw new IllegalArgumentException("PublikationInitial \"" + id + "\" wird nicht untersttzt");
    }
    return publikationInitial;
  }  
  
  /**
   * Programmeinstieg RDS/TMC.<br>
   * 
   * @param arguments Kommandozeilenargumente, folgende speziellen Argumente werden ausgewertet:<p>
   * -netz: Netz, dessen Infrastrukturobjekte verwendet werden.
   * -lms: Zustndige Landesmeldestelle (LMS). 
   */
  public static void main(String[] arguments)
  {
     if (_uehDebug == null)
     {
       _uehDebug = Debug.getLogger();
     }
    
     Thread.setDefaultUncaughtExceptionHandler(
        new UncaughtExceptionHandler()
        {
          public void uncaughtException(Thread t, Throwable e)
          {
            _uehDebug.error("Uncaught Exception " + e + "in Thread " + t +":\n", e);
          }
        }
    );
    
    StandardApplicationRunner.run(new RDSTMCApp(), arguments);
  }
  
  /**
   * Debug-Logger fr Logging-Ausgaben des UncaughtExceptionHandlers
   */
  private static Debug _uehDebug = null;
  
  /***************************************************************************/
  /* Implementierung Interface StandardApplication                           */
  /***************************************************************************/
  
  /**
   * berschriebene Methode von <code>StandardApplication</code>, 
   * die die speziellen Startparameter auswertet.<br>
   * Die Argumtwerte, die ausgewertet werden, sind durch die Attribute {@link #_netzPid} 
   * und {@link #_lmsPid} spezifiziert
   * 
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();
   
    // Pflichtargumete
    _netzPid = argumentList.fetchArgument("-netz=").asNonEmptyString();
    _lmsPid = argumentList.fetchArgument("-lms=").asNonEmptyString();
    
     // Optionale Argumente
    _konfigBereichPid = argumentList.fetchArgument("-konfigBereich=").asString();
    _locationMethodeId = argumentList.fetchArgument("-locationMethode=0").intValueBetween(0, 1);
    _publikationInitialId = argumentList.fetchArgument("-publikationInitial=0").intValueBetween(0, 1);
    
    _idOrganisation = argumentList.fetchArgument("-idOrganisation=").asString();
    _idNummer = argumentList.fetchArgument("-idNummer=").asString();
    _ablaufOffset = argumentList.fetchArgument("-ablaufOffset=120").intValueBetween(0, 144000);
    
    argumentList.fetchUnusedArguments();
  }
      
  /**
   * berschriebene Methode von StandardApplication, die die Initialisierung durchfhrt.<br>
   * Es wird eine Instanz von <code>VerwaltungEngstellen</code> erzeugt und initialisiert
   * 
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;
    
//    Thread.setDefaultUncaughtExceptionHandler(
//        new UncaughtExceptionHandler()
//        {
//          public void uncaughtException(Thread t, Throwable e)
//          {
//            _uehDebug.error("Uncaught Exception " + e + "in Thread " + t +":\n", e);
//          }
//        }
//    );
    
    try
    {
      MessageSender.getInstance().setApplicationLabel(
          connection.getLocalApplicationObject().getName());

      // Initialisiere die Objekte der Daten des Interfaces MeldungenGlobal
      _locationMethode = getLocationMethode(_locationMethodeId); 
      _publikationInitial = getPublikationInitial(_publikationInitialId);
      _konfigBereich = getKonfigBereich(_konfigBereichPid);
      _debug.config("Neue dynamische Objekte werden im Konfigurationsbereich " + _konfigBereich.getPid() + " abgelegt");
      
      // Bestimme bentigte Konfigurationsobjekte
      ConfigurationObject netz = getKonfigObjekt(_netzPid, "typ.netz");
      ConfigurationObject lms = getKonfigObjekt(_lmsPid, "typ.landesMeldeStelle");
      
      // Initialisiere Konfigurationsobjekte des/der Netzes/Netze
      Konfiguration konfiguration = new Konfiguration(connection, netz, true, true);
      
      // Initialisiere Verwaltung Modell-Parameter der LMS
      RDSParameterVerwaltung parameterVerwaltung = new RDSParameterVerwaltung(connection, lms);
      if (!parameterVerwaltung.wartenAufParameter(TIMEOUT))
      {
        throw new IllegalStateException("Keine Parameterdaten empfangen nach " + TIMEOUT + "ms");
      }
               
      // instanziiere Meldungserstellung
      _meldungserstellung = new Meldungserstellung(connection, konfiguration, parameterVerwaltung, this, false); 
      
      _debug.info("SWE RDS/TMC initialisiert");
    }
    catch (Exception e)
    {
      _debug.error("Abbruch der SWE RDS/TMC whrend der Initialisierung", e);
      System.exit(1);
    }
  }
  
  /***************************************************************************/
  /* Implementierung Interface MeldungenGlobal                               */
  /***************************************************************************/
  
  /**
   * @return den Konfigurationsbereich, in dem dynamische Meldungsobjekte abgelegt werden
   */
  public ConfigurationArea getKonfigurationsbereich()
  {
    return _konfigBereich;
  }
  
  /**
   * @return die LocationMethode, nach der Meldungsdatenstze kodiert werden
   */
  public LocationMethode getLocationMethode()
  {
    return _locationMethode;
  }
  
  /**
   * @return die Methode, nach der beim Programmstart aktuelle Zustnde publiziert werden
   */
  public PublikationInitial getPublikationInitial()
  {
    return _publikationInitial;
  }
  
  /**
   * @return die (Text-) Id der Organisationseinheit, die einer generierten Meldung zugeordnet sein soll
   * - i. d. R. LMS
   */
  public String getIdOrganisation()
  {
    return _idOrganisation;
  }
  
  /**
   * @return die (numerische) Id der Organisationseinheit, die einer generierten Meldung zugeordnet sein soll
   * - i. d. R. LMS
   */
  public String getIdNummer()
  {
    return _idNummer;
  }
   
  /**
   * @return Zeitspanne in min., wielange eine Meldung gltig (nicht abgelaufen)
   * sein soll, nachdem sie aktualisiert wurde
   */  
  public int getAblaufOffset()
  {
    return _ablaufOffset;
  }
  
}
